function [a, b, c, vare, P, fit] = arIdent(y,u,yv,uv,na,nb,nc,nk,sp)
// ARX or ARMAX estimation from a given range of number of parameters.
//
// Calling Sequence
// [a, b, vare, P, fit] = arIdent(y,u,yv,uv,na,nb,nc,nk,sp)   
//
// Parameters
//   Inputs:
//           y: vector of output data used for identification
//           u: vector of input data used for identification
//           yv: vector of output validation data
//           uv: vector of input validation data
//           na: vector of numbers of poles
//           nb: vector of numbers of zeros + 1
//           nc: vector of noise filter numerator polynomial lengths - 1
//           nk: vector of time delays from input to output
//           sp: selects identification criterion: sp = 1 -> simulation, sp = 0 -> prediction (faster to compute)
//         
//   Outputs:
//            a: coeficients of a(q) of the best estimate
//            b: coeficients of b(q) of the best estimate
//            vare: estiamted variance of e with the best estimate
//            P: normed variance of the best estimate, i.e. P = P_true / vare.
//            fit: fit factor achieved by the best model on validation data yv, uv.
//
// Description
// Computes the best ARX or ARMAX estimate from a given range of number of parametrs. An ARX model is evaluated for zero values in the nc vector, ARMAX otherwise. Fitfactor defined as fit = 1 - ||yv-ys||_2 / ||yv-mean(yv)||_2 is used to compare models' responses.
//
//Standard structure of ARMAX (or ARX) model is used:
//
//      a(d)y(t) = b(d)u(t) + c(d)e(t), where
//
//      a(d) = 1 + a_1*d + ... + a_na*d^na  
//
//      b(d) = b_0*d^nk + ... + b_nb * d^(nk+nb)
//
//      c(d) = 1 + c_1*d + ... + c_nc * d^nc,
//
//with d being the unit delay operator: dx(t) = x(t-1). c(d) = 1 for ARX
//
// Examples
//data = read('.../identification/demos/dryer.dat',-1,2); //data = [u y]
//y = data(1:400,2); u = data(1:400,1); //identification data
//yv = data(401:$,2); uv = data(401:$,1); //validation data
//[a, b, vare, P, fit] = arIdent(y,u,yv,uv,1:4,1:4,0,0:3,0); //prediction criterion is used
//[fit, yp] = valPred(a, b, [], yv, uv);
//plot(yv);
//plot(yp,'-r');   
//
// See also
//  arx
//  rarx
//  valPred
//  valSim
//  subid
//  subspaceIdent

  [LHS RHS] = argn(0);
  if (RHS ~= 9)
    error('Wrong number of input arguments. Use [a, b, c, vare, P, fit] = arIdent(y,u,yv,uv,na,nb,nc,nk,sp).');
  end
  if (sp ~= 0 & sp ~= 1)
    error('identification criterion selection error: use sp = 1 for simulation and sp = 0 for prediction');
  end
  bestFit = -number_properties("huge");
  besta = []; bestb = []; bestVare = []; bestP = []; bestC = [];
  for q = nc
  for i = na
    for j = nb
      for k = nk
        if (q == 0)
          [a b vare P] = arx(y,u,i,j,k);
          c = 1;
        else
          [a b c vare P] = armax(y,u,i,j,q,k);
        end
        if (sp == 1) //simulation
          fit = valSim(a,b,yv,uv);
          if (fit > bestFit)
            bestFit = fit;
            besta = a; bestb = b; bestVare = vare; bestP = P; bestC = c;
          end
        else //prediction
          fit = valPred(a,b,c,yv,uv);
          if (fit > bestFit)
            bestFit = fit;
            besta = a; bestb = b; bestVare = vare; bestP = P; bestC = c;
          end
        end       
      end
    end
  end
  end
  a = besta; b = bestb; vare = bestVare; P = bestP; c = bestC;
  fit = bestFit;
endfunction