function [a, b, c, vare, P, nu] = armax(y, u, na, nb, nc, nk, phi, theta0, P0, nu0, vare0, eps0)
//  ARMAX model estimation.
//
// Calling Sequence
//   [a, b, c, vare] = armax(y,u,na,nb,nc) //nk = 0, phi = 1
//   [a, b, c, vare] = armax(y,u,na,nb,nc,nk) //phi = 1
//   [a, b, c, vare] = armax(y, u, na, nb, nc, nk, phi)
//   [a, b, c, vare, P, nu, eps] = armax(y, u, na, nb, nc, nk, phi, theta0, P0)
//   [a, b, c, vare, P, nu, eps] = armax(y, u, na, nb, nc, nk, phi, theta0, P0, nu0, vare0, eps0)
//
// Parameters
//   Inputs:
//           y: vector of outputs
//           u: vector of inputs
//           na: number of poles = [number of coefficients of a(d)] - 1
//           nb: number of zeros + 1 = [number of coefficients of b(d)] - nk
//           nc: number of coefficients of the noise filter numerator - 1
//           nk: input-output time delay
//           theta0: initial estimate of the polynomials a(q), b(q): theta0 = [a0 b0 c0]
//           P0: The covarience matrix of the initial estimate of [-a0($:-1:2), b0($:-1:1), c0($:-1:1)]
//           nu0: initial index in the recursion - acts as a weight of the vare0 estimate
//           vare0: Initial estimate of the noise varience
//           phi: exponential forgetting factor: 1 -> no forgetting, common choice: phi in [0.9, 1]
//           eps0: estimates of the prediction error 
//   Outputs:
//            a: coeficients of a(d)
//            b: coeficients of b(d)
//            c: coeficients of c(d)
//            vare: the varience of e
//            P: normed covarience matrix of the estimate, i.e. P_true / vare.
//            nu: index in the recursion
//            eps: the last prediction error
//
// Description
//   Computes recursively an armax model of a given order, and the input noise varience. An initial estimate theta0, P0 and the forgetting factor phi are optinal parameters. The optional parameters nu0, vare0, eps0 can be used for recursive identification.
//
//Standard armax model structure is used:
//
//      a(d)y(t) = b(d)u(t) + c(d)e(t), where
//
//      a(d) = 1 + a_1*d + ... + a_na*d^na  
//
//      b(d) = b_0*d^nk + ... + b_nb * d^(nk+nb)
//
//      c(d) = 1 + c_1*d + ... + c_nc * d^nc,
//
//      with d being the unit delay operator: dx(t) = x(t-1).
//
//The length of the input data vectors u, y must be at least max(na+1, nb+nk).
//
//
// Examples
//data = read('.../identification/demos/dryer.dat',-1,2); //data = [u y]
//y = data(1:400,2); u = data(1:400,1); //identification data
//yv = data(401:$,2); uv = data(401:$,1); //validation data
//[a b c vare P] = armax(y,u,3,3,2,2);
//[fit, yp] = valPred(a, b, c, yv, uv);
//plot(yv);
//plot(yp,'-r');
//
// See also
//  arx
//  rarx
//  arIdent
//  validate
//  valPred
//  valSim
//  subid
//  subspaceIdent


  [LHS RHS] = argn(0);
  if (RHS < 5)
    error('Not enough input arguments, at least five arguments are needed: y, u, na, nb, nc');
  end
  if (RHS == 5)
    nk = 0;
  end
  if (RHS == 8)
    error('The covariance matrix of theta0 must be specified.')
  end
  if (RHS <= 7)
    th = zeros(na+nb+nc,1);
    P = 100*eye(size(th,1), size(th,1));
  end
  if (RHS >= 9)
    th = [mtlb_fliplr(-theta0(2:na+1))'; mtlb_fliplr(theta0(na+nk+2:na+nk+nb+1))'; mtlb_fliplr(theta0(na+nk+nb+2:$))'];
    P = P0;
  end 
  if (RHS == 10)
    error('An initial estimate of the noise varience and prediction errors must be provideed with nu0.')
  end
  if (RHS >= 11)
    nu = nu0;
    nus2 = nu*vare0;
  end
  if (RHS <= 9)
    nu = 0;
    nus2 = nu*4; //vare0 = 4
  end
  if(RHS < 7)
    phi = 1;
  end
  if (RHS < 12)
    eps = zeros(nc,1); 
  else
    eps = eps0;
  end
  
  if (size(y,1) ~= size(u,1) | size(y,2) ~= size(u,2))
    error('y and u must be of the same size.')
  end
  if (size(y,1) < size(y,2))
    y = y';
    u = u';
  end
  
   
//-----------------Compute the estimate------------------------
  for t = max(na+1, nb+nk) : size(y,1)
    z = [y(t-na:t-1,1)' u(t-nb-nk+1:t-nk,1)' eps($-nc+1:$,1)']';
    nus2 = nus2 + (y(t) - z'*th)^2/(1 + z'*P*z);
    P = (1/phi) * (P - P*z*z'*P/(1 + z'*P*z));
    th = th + P*z*(y(t)-z'*th);
    nu = nu + 1;
    eps = [eps(2:$);(y(t)-z'*th)];
  end
  a = [1 -mtlb_fliplr(th(1:na)')];
  b = [zeros(1,nk) mtlb_fliplr(th(na+1:na+nb)')];
  c = [1 mtlb_fliplr(th(na+nb+1:$)')];
  vare = nus2 / nu;
 endfunction