function [A, B, Data,vare, P] = arx(y,u,na,nb,nk)
// ARX model estimation.
//
// Calling Sequence
//  [a, b, vare] = arx(y,u,na,nb) //nk = 0 here
//  [a, b, vare] = arx(y,u,na,nb,nk)
//
// Parameters
//   Inputs:
//           y: vector of outputs
//           u: vector of inputs 
//           na: number of poles
//           nb: number of zeros + 1
//           nk: time delay from input to output
//           
//   Outputs:
//            a: coeficients of a(d)
//            b: coeficients of b(d)
//            vare: the varience of e
//            P: normed covarience matrix of the estimate, i.e. P_true / vare.
//
// Description
// [a, b, vare] = arx(y,u,na,nb) computes the coefficients of the arx model polynomials a(d), b(d) and noise varience                                vare given na, nb and optional nk.
//
//The coefficients of a(d), b(d) are returned as row vectors for SISO case and p x p x (na+1), p x m x (nb+nk) matrices otherwise.
//
//Standard structure of the arx model is used:
//
//a(d)y(t) = b(d)u(t) + e(t), where
//
//a(d) = 1 + a_1*d + a_2*d^2 + ... + a_na*d^na,
//
//b(d) = b_0*d^(nk) + b_1*d^(nk+1) + ... +  b_1*d^(nk+nb),
//
//with d being the unit delay operator: dx(t) = x(t-1).
//
// Examples
//data = read('.../identification/demos/dryer.dat',-1,2); //data = [u y]
//y = data(1:400,2); u = data(1:400,1); //identification data
//yv = data(401:$,2); uv = data(401:$,1); //validation data
//[a b vare] = arx(y,u,3,3,2);
//[fit, ys] = valSim(a, b, yv, uv);
//plot(yv);
//plot(ys,'-r');
//
// See also
//  armax
//  rarx
//  arIdent
//  valPred
//  valSim
//  subid
//  subspaceIdent

// CODE \\


  [LHS RHS] = argn(0);
  if (RHS < 4)
    error('Not enough input arguments. Use [a, b, vare] = arx(y,u,na,nb [,nk]).');
  end
  if (RHS == 4 )
    nk = 0;
  end
  if (size(y,1) ~= size(u,1) | size(y,2) ~= size(u,2))
    error('y and u must be of the same size.')
  end
  if (size(y,1) < size(y,2))
    y = y';
    u = u';
  end
  p = size(y,2);
  m = size(u,2);
  //------------Construct the regression matrix------------
  N = size(y,1);
  Data = zeros(N-max(na+1,nb+nk)+1, p*na+m*nb);
  vecY = y(:); vecU = u(:);
  for q = 1:na
    i = na-q+1;
    Data(:, p*(i-1)+1 : i*p) = y(max(na,nb+nk-1)-i+1 : N-i,:);
  end
  for q = 1:nb
    i = nb-q+1;
    Data(:, m*(i-1)+1+na*p : m*i+na*p) = u(max(na,nb+nk-1)-i-nk+2 : N-i-nk+1,:);
  end
  //------------------Solve LS------------------------------
  theta = Data \ y(max(na+1,nb+nk):$,:);
  a = [eye(p,p) -theta(1:p*na,:)'];
  b = [zeros(p,nk*m) theta(p*na+1:$,:)'];
  if (p*m > 1)//MIMO
    A = zeros(p,p,na+1);
    for i = 1:na+1
      A(:,:,i) = a(:,(i-1)*p+1:i*p);
    end
    B = zeros(p,m,nb+nk);
    for i=1:nb+nk
      B(:,:,i) = b(:,(i-1)*m+1:i*m);
    end
  else // SISO
    A = a;
    B = b;
  end
  vare = mean((y(max(na+1,nb+nk):$,:) - Data*theta).^2);
  P = inv(Data'*Data); 
//end
  
endfunction

