function [A,B,C,D,K,R,x0,fit,fitTable] = subspaceIdent(y,u,yv,uv,i,n,sp,dmat,stabCheck)
//Subspace identification in a given range of parameters
//
// Calling Sequence
//    [A,B,C,D,K,R,fit,fitTable] = subspaceIdent(y,u,yv,uv,i,n,sp,dmat)
//    [A,B,C,D,K,R,fit,fitTable] = subspaceIdent(y,u,yv,uv,i,n,sp,dmat,stabCheck)
//
// Parameters
//   Inputs:
//           y: matrix of measured outputs used for identification
//           u: matrix of measured inputs used for identification
//           yv: validation output data
//           uv: validation input data
//           sp: sp = 1 -> simulation, sp = 0 -> prediction
//           i: vector of the sizes of block-hankel matrices used in the algorithm
//           n: vector of orders to iterate over
//           dmat: dmat = 1 for nonzero matrix D, dmat = 0 for zero matrix D
//           stabCheck: = 1 -> returns a stable model only (if found). The value in the fitTable is NaN for unstable                           models and stabCheck = 1. Default: stabCheck = 0.
//   Outputs:
//           A,B,C,D,K,R: Combined state space system matrices
//           fit: fit factor for the best model
//           fitTable: table of fitFactors for every evaluated combinatin of the free parameters
//           x0: initial state estimate
//
// Description
//   Computes the best model in the range of free parameters using a N4SID subpsace identification algorithm.
//   The free parameters are the model order n and the block size i. Optimization criterion can be either prediction or      simulation error on the validation data.
//
//Uses combined "robust" algorith implemented in subid function.
//
//           Combined state space system:
//           x_(k+1) = A x_k + B u_k + K e_k;
//
//           y_k = C x_k + D u_k + e_k;
//
//           cov(e_k) = R
//
//
//The model is estimated only for the combinations (i,n) for which i >= n; NaN is written into the fitTable when i < n.
//
//Examples
//data = read('.../identification/demos/dryer.dat',-1,2); //data = [u y]
//y = data(1:400,2); u = data(1:400,1); //identification data
//yv = data(401:$,2); uv = data(401:$,1); //validation data
//[A,B,C,D,K,R,x0,fit fitTable] = subspaceIdent(y,u,yv,uv,[6:8],[2:4],0);
//[yp x] = lsim(A-K*C,[K B-K*D],C,[zeros(size(D,1),size(yv,2)) D],[yv';uv'],x0); // prediction
//plot(yv);
//plot(yp,'-r')
//
// See also
//  subid
//  arx
//  rarx
//  arIdent
//  valPred
//  valSim
// 
//
//Bibliography
//           Overschee, DeMoor: Subspace identification for linear systems
//           Stochastic algorithm:   Figure 3.13 page 90 (positive)
//           Combined algorithm:     Figure 4.8 page 131 (robust)
//
  [LHS RHS] = argn(0);
  if (RHS ~= 9 & RHS ~= 8) then
    error('Wrong number of input arguments. Use subspaceIdent(y,u,yv,uv,i,n,sp,dmat) or subspaceIdent(y,u,yv,uv,i,n,sp,dmat,stabCheck).');
  end
  if (size(yv,2) < size(yv,1)) then
    yv = yv';
  end
  if (size(uv,2) < size(uv,1)) then
    uv = uv';
  end
  if (exists('stabCheck') == 0) then
    stabCheck = 0;
  end
  
  bestFit = -number_properties("huge");
  bestA = []; bestB = []; bestC = []; bestD = [];
  bestK = []; bestRo = []; bestx0 = [];
  fitTable = %nan*ones(i($)*size(yv,2),n($));
  for nn=n 
    for ii=i
      if (nn > ii * size(yv,1))
        continue;
      end
      [A, B, C, D, K, R] = subid(y,u,ii,dmat,nn,[],[],1);
      if (stabCheck == 1 & max(abs(spec(A))) >= 1)
        continue;
      end
      x0 = estimatex0(yv,uv,A,B,C,D);
      if (sp == 1) //simulation
        fit = fitFactor(lsim(A,B,C,D,uv,x0), yv);
        fitTable((ii-1)*size(yv,2)+1:ii*size(yv,2),nn) = fit;
        if (sum(fit)/size(fit,1) > bestFit) then
          bestFit = fit;
          bestA = A; bestB = B; bestC = C; bestD = D;
          bestK = K; bestR = R; bestx0 = x0;
        end
      else //prediction
        fit = fitFactor(lsim(A-K*C,[K B-K*D],C,[zeros(size(D,1),size(yv,1)) D],[yv;uv],x0), yv);
        fitTable((ii-1)*size(yv,2)+1:ii*size(yv,2),nn) = fit;
        if (sum(fit)/size(fit,1) > bestFit) then
          bestFit = fit;
          bestA = A; bestB = B; bestC = C; bestD = D;
          bestK = K; bestR = R; bestx0 = x0;
        end
      end       
    end
  end
  A = bestA; B = bestB; C = bestC;
  K = bestK; R = bestR; x0 = bestx0;
  fit = bestFit;
endfunction