function [fit, yp] = valPred(a, b, c, yv, uv)
// ARX or ARMAX model prediction validation.
//
// Calling Sequence
//  [fit, ys] = valPred(a, b, c yv, uv)
//
// Parameters
//   Inputs:
//    a: coefficients of the a(d) polynomial of an ARX or ARMAX model
//    b: coefficients of the b(d) polynomial of an ARX or ARMAX model
//    c: coefficients of the c(d) polynomial of an ARMAX model. Use c = [] for ARX.
//    yv: output validation data
//    uv: input validation data
// 
//   Outputs:
//    fit: fit factor
//    yp: predicted output
//
// Description
//Produces a fitfactor and the predicted output yp of an ARX or ARMAX model based on the validation data yv, uv.
//
//The polynomial matrices A(d), B(d) are p x p x (na+1), p x m x (nb+nk) for MIMO ARX or row vectors for SISO ARX or ARMAX (which is SISO only).
//
//Here, the fitfactor is defined as fit = 1 - ||yv-ys||_2 / ||yv-mean(yv)||_2
//
//
// Examples
//data = read('.../identification/demos/dryer.dat',-1,2); //data = [u y]
//y = data(1:400,2); u = data(1:400,1); //identification data
//yv = data(401:$,2); uv = data(401:$,1); //validation data
//[a b c vare P] = armax(y,u,3,3,2,2);
//[fit, yp] = valPred(a, b, c, yv, uv);
//plot(yv);
//plot(yp,'-r');
//
// See also
//  arx
//  rarx
//  armax
//  arIdent
//  validate
//  valSim
//  subid
//  subspaceIdent

  [LHS RHS] = argn(0);
  if (RHS ~= 5)
    error('Wrong number of input arguments. Use [fit, yp] = valPred(a, b, c, yv, uv).');
  end
  if (size(yv,1) < size(yv,2))
    yv = yv';
  end
  if (size(uv,1) < size(uv,2))
    uv = uv';
  end
  if (size(yv,1) ~= size(uv,1) | size(yv,2) ~= size(uv,2))
      error('uv, yv must be of the same size.')
  end
    
  A = a; B = b;
  if (size(size(a),2) == 2) //SISO
    na = size(a,1)*size(a,2) - 1;
    nb = size(b,1)*size(b,2);
    p = 1; m = 1;
  else //MIMO
    p = size(A,1); m = size(B,2);
    na = size(A,3) - 1;
    nb = size(B,3);
  end
  nc = size(c,1)*size(c,2) - 1;
  
  if (isempty(c) | nc == 0) //ARX, much faster
    N = size(yv,1);
    Data = zeros(N-max(na+1,nb)+1, p*na+m*nb);
    for q = 1:na
      i = na-q+1;
      Data(:, p*(i-1)+1 : i*p) = yv(max(na,nb-1)-i+1 : N-i,:);
    end
    for q = 1:nb
      i = nb-q+1;
      Data(:, m*(i-1)+1+na*p : m*i+na*p) = uv(max(na,nb-1)-i+2 : N-i+1,:);
    end
    if (p*m > 1)
      th = [];  
      for i=1:na
        th = [th -A(:,:,i+1)];  
      end
      for i=1:nb
        th = [th B(:,:,i)];  
      end
    else
      th = [(-a(2:$))  b];
    end
    yp = Data * th';
    yp = [yv(1:max(na+1,nb)-1,:); yp]
    fit = fitFactor(yp, yv);
  else //ARMAX - SISO only
    yp = yv(1:max(na+1,nb),1);
    eps = zeros(nc,1);
    for i = max(na+1,nb)+1 : size(yv,1)
      yp = [yp; yv(i-na:i-1)'*-a($:-1:2)' + uv(i-nb+1:i)'*b($:-1:1)' + c(2:$)*eps ];
      eps = [yv(i) - yp($); eps(1:$-1)];
    end
    fit = fitFactor(yp, yv);
  end
yp = yp';  
endfunction