function [fit, ys] = valSim(a, b, yv, uv)
// ARX or ARMAX model simulation validation.
//
// Calling Sequence
//  [fit, ys] = valSim(a, b, yv, uv)
//
// Parameters
//   Inputs:
//    a: coefficients of the a(d) polynomial (matrix) of an ARX or ARMAX model
//    b: coefficients of the b(d) polynomial (matrix) of an ARX or ARMAX model
//    yv: output validation data
//    uv: input validation data
// 
//   Outputs:
//    fit: fit factor
//    ys: simulated output
//
// Description
//Produces a fitfactor and the simulated output ys of an ARX or ARMAX model based on the validation data yv, uv.
//
//The polynomial matrices A(d), B(d) are p x p x (na+1), p x m x (nb+nk) for MIMO systems or row vectors for SISO.
//
//The c polynomial of an ARMAX model is not needed for the simulation.
//
//Here, the fitfactor is defined as fit = 1 - ||yv-ys||_2 / ||yv-mean(yv)||_2
//
//
// Examples
//data = read('.../identification/demos/dryer.dat',-1,2); //data = [u y]
//y = data(1:400,2); u = data(1:400,1); //identification data
//yv = data(401:$,2); uv = data(401:$,1); //validation data
//[a b vare] = arx(y,u,3,3,2);
//[fit, ys] = valSim(a, b, yv, uv);
//plot(yv);
//plot(ys,'-r'); 
//
// See also
//  arx
//  rarx
//  armax
//  arIdent
//  validate
//  valPred
//  subid
//  subspaceIdent
  [LHS RHS] = argn(0);
  if (RHS ~= 4)
    error('Wrong number of input arguments. Use [fit, ys] = valSim(a, b, yv, uv).');
  end
  if (size(yv,1) < size(yv,2))
    yv = yv';
  end
  if (size(uv,1) < size(uv,2))
    uv = uv';
  end
  if (size(yv,1) ~= size(uv,1) | size(yv,2) ~= size(uv,2))
      error('uv, yv must be of the same size.')
  end
    
  if (size(size(a),2) == 2) //SISO
    na = size(a,1)*size(a,2) - 1;
    nb = size(b,1)*size(b,2);
    ys = yv(1:max(na+1,nb),:);
    for i = max(na+1,nb)+1 : size(yv,1)
      ys = [ys; -ys(i-na:i-1)'*a($:-1:2)' + uv(i-nb+1:i)'*b($:-1:1)'];
    end
  else //MIMO
    A = a; B = b;
    p = size(A,1); m = size(uv,2); 
    na = size(A,3)-1; nb = size(B,3);
    ys = yv(1:max(na,nb),:);
    for i = 1+max(na,nb) : size(uv,1)
      yi= zeros(p,1);
      for j=1:na
        yi = yi - A(:,:,j+1) * ys(i-j,:)'; 
      end
      for j=1:nb
        yi = yi + B(:,:,j) * uv(i-j+1,:)'; 
      end
      ys = [ys;yi'];
    end
  end
  fit = fitFactor(ys, yv);
ys = ys';  
endfunction