function [fit, y] = validate(v1,v2,v3,v4,v5,v6,v7,v8)
// Model validation
//
// Calling Sequence
//  [fit, y] = validate(A,B,C,D,yv,uv,sp)
//  [fit, y] = validate(A,B,C,D,K,yv,uv,sp)
//  [fit, y] = validate(a,b,c,yv,uv,sp)
//  [fit, y] = validate(a,b,yv,uv,sp)
//
// Parameters
//   Inputs:
//    A,B,C,D: state-space matrices
//    a: coefficients of the a(d) polynomial (matrix) of an ARX or ARMAX model
//    b: coefficients of the b(d) polynomial (matrix) of an ARX or ARMAX model
//    c: coefficients of the c(d) polynomial of an ARMAX model
//    yv: output validation data
//    uv: input validation data
//    sp: =1 -> simulation, =0 -> prediction
// 
//   Outputs:
//    fit: fit factor
//    y: simulated / predicted output
//
// Description
//Produces the fitfactor and the simulated or predicted output of an ARX, ARMAX or a state-space model based on the validation data yv, uv.
//
//The polynomial matrices a(d), b(d) are p x p x (na+1), p x m x (nb+nk) for MIMO ARX or row vectors for SISO ARX or ARMAX (which is SISO only).
//
//The c polynomial of an ARMAX model and the Kalman gain K for a state space model are not needed for a simulation.
//
//Here, the fitfactor is defined as fit = 1 - ||yv-y||_2 / ||yv-mean(yv)||_2
//
//
// Examples
//data = read('.../identification/demos/dryer.dat',-1,2); //data = [u y]
//y = data(1:400,2); u = data(1:400,1); //identification data
//yv = data(401:$,2); uv = data(401:$,1); //validation data
//[a b c vare P] = armax(y,u,3,3,2,2);
//[fits, ys] = validate(a, b, c, yv, uv, 1); //simulation
//[fitp, yp] = validate(a, b, c, yv, uv, 0); //prediction
//plot(yv);
//plot(yp,'--g');
//plot(ys,'-r');
//
// See also
//  arx
//  rarx
//  armax
//  arIdent
//  valPred
//  subid
//  subspaceIdent

  [LHS RHS] = argn(0);
  if (RHS ~= 5 & RHS ~= 6 & RHS ~= 7 & RHS ~= 8 )
    error('Wrong number of input arguments. Use [fit, y] = validate(A,B,C,D,yv,uv,sp) or [fit, y] = validate(a,b,c,yv,uv,sp) or [fit, y] = validate(a,b,yv,uv,sp)');
  end
  if (RHS == 5)
    sp = v5;
  end
  if (RHS == 6)
    sp = v6;
  end
  if (RHS == 7)
    sp = v7;
  end
  if (RHS == 8)
    sp = v8;
  end
  if (sp ~= 0 & sp ~= 1)
    error('identification criterion selection error: use sp = 1 for simulation and sp = 0 for prediction')
  end    
    
  if (RHS == 5)//Polynomials a,b given
    a = v1; b = v2; yv = v3; uv = v4;
    if (sp == 1)//simulation
      [fit, y] = valSim(a,b,yv,uv);
    else//prediction
      [fit, y] = valPred(a,b,[],yv,uv);
    end
  end
  
  if (RHS == 6)//Polynomials a,b,c given
    a = v1; b = v2; c = v3; yv = v4; uv = v5;
    if (sp == 1)//simulation
      [fit, y] = valSim(a,b,yv,uv);
    else//prediction
      [fit, y] = valPred(a,b,c,yv,uv);
    end
  end
  
  if (RHS == 7 | RHS == 8)//System matrices A,B,C,D and/or K given
    A = v1; B = v2; C = v3; D = v4;
    try
      [A B; C D];
    catch
      error('Incompatible dimensions of A,B,C,D')
    end
    if (RHS == 7 & sp == 0)
      error('Kalman gain matrix K must be provided for prediction.')
    end
    if (RHS == 8)
      K = v5;
      if (size(K,1) ~= size(A,1) | size(K,2) ~= size(C,1))
        error('Incompatible size of K.')
      end
      yv = v6; uv = v7;//RHS = 8
    else
      yv = v5; uv = v6; // RHS = 7
    end
     
    if (size(yv,2) < size(yv,1))
      yv = yv';
    end
    if (size(uv,2) < size(uv,1))
      uv = uv';
    end
    if (size(yv,1) ~= size(uv,1) | size(yv,2) ~= size(uv,2))
      error('uv, yv must be of the same size.')
    end
    
    x0 = estimatex0(yv,uv,A,B,C,D);
    if (sp == 1)//simulation
      y = lsim(A,B,C,D,uv,x0)
      fit = fitFactor(y,yv);
    else//prediction
      y = lsim(A-K*C,[K B-K*D],C,[zeros(size(D,1),size(yv,1)) D],[yv;uv],x0);
      fit = fitFactor(y,yv); 
    end
  end
endfunction